@@ -1211,6 +1211,32 @@ gnm_conf_get_core_gui_editing_hotkey_node (void)
 }
 
 #ifdef GNM_USE_HILDON
+static struct cb_watch_bool watch_core_gui_editing_showcursor = {
+	0, "core/gui/editing/showcursor", FALSE,
+};
+
+gboolean
+gnm_conf_get_core_gui_editing_showcursor (void)
+{
+	if (!watch_core_gui_editing_showcursor.handler)
+		watch_bool (&watch_core_gui_editing_showcursor);
+	return watch_core_gui_editing_showcursor.var;
+}
+
+void
+gnm_conf_set_core_gui_editing_showcursor (gboolean x)
+{
+	if (!watch_core_gui_editing_showcursor.handler)
+		watch_bool (&watch_core_gui_editing_showcursor);
+	set_bool (&watch_core_gui_editing_showcursor, x);
+}
+
+GOConfNode *
+gnm_conf_get_core_gui_editing_showcursor_node (void)
+{
+	return get_node (watch_core_gui_editing_showcursor.key);
+}
+
 static struct cb_watch_bool watch_core_gui_editing_portraitmode = {
 	0, "core/gui/editing/portraitmode", FALSE,
 };
