@@ -160,6 +160,8 @@ item_bar_calc_size (ItemBar *ib)
 	item_list = pango_itemize (context, "A", 0, 1, attr_list, NULL);
 	pango_attr_list_unref (attr_list);
 
+	if (ib->pango.item)
+	  pango_item_free (ib->pango.item);
 	ib->pango.item = item_list->data;
 	item_list->data = NULL;
 
@@ -240,11 +242,10 @@ ib_draw_cell (ItemBar const * const ib, cairo_t *cr,
 	      ColRowSelectionType const type,
 	      char const * const str, GocRect *rect)
 {
-	GtkLayout *canvas = GTK_LAYOUT (ib->base.canvas);
-	GtkWidget *widget = GTK_WIDGET (canvas);
-	GtkStyle *style = gtk_widget_get_style (widget);
-	PangoFont *font;
-	PangoRectangle size;
+	GtkLayout	*canvas = GTK_LAYOUT (ib->base.canvas);
+	GtkWidget   *widget = GTK_WIDGET (canvas);
+	PangoFont	*font;
+	PangoRectangle   size;
 	GOColor color, font_color;
 	int shadow, ascent;
 
@@ -253,24 +254,24 @@ ib_draw_cell (ItemBar const * const ib, cairo_t *cr,
 	case COL_ROW_NO_SELECTION:
 		shadow = GTK_SHADOW_OUT;
 		font   = ib->normal_font;
-		color = GO_COLOR_FROM_GDK (style->bg[GTK_STATE_NORMAL]);
-		font_color = GO_COLOR_FROM_GDK (style->fg[GTK_STATE_NORMAL]);
+		color = GO_COLOR_FROM_GDK (widget->style->bg[GTK_STATE_ACTIVE]);
+		font_color = GO_COLOR_FROM_GDK (widget->style->fg[GTK_STATE_ACTIVE]);
 		ascent = ib->normal_font_ascent;
 		break;
 
 	case COL_ROW_PARTIAL_SELECTION:
 		shadow = GTK_SHADOW_OUT;
 		font   = ib->bold_font;
-		color = GO_COLOR_FROM_GDK (style->bg[GTK_STATE_SELECTED]);
-		font_color = GO_COLOR_FROM_GDK (style->fg[GTK_STATE_SELECTED]);
+		color = GO_COLOR_FROM_GDK (widget->style->light[GTK_STATE_ACTIVE]);
+		font_color = GO_COLOR_FROM_GDK (widget->style->fg[GTK_STATE_PRELIGHT]);
 		ascent = ib->bold_font_ascent;
 		break;
 
 	case COL_ROW_FULL_SELECTION:
 		shadow = GTK_SHADOW_IN;
 		font   = ib->bold_font;
-		color = GO_COLOR_FROM_GDK (style->dark[GTK_STATE_SELECTED]);
-		font_color = GO_COLOR_FROM_GDK (style->fg[GTK_STATE_SELECTED]);
+		color = GO_COLOR_FROM_GDK (widget->style->light[GTK_STATE_ACTIVE]);
+		font_color = GO_COLOR_FROM_GDK (widget->style->fg[GTK_STATE_NORMAL]);
 		ascent = ib->bold_font_ascent;
 		break;
 	}
@@ -290,7 +291,7 @@ ib_draw_cell (ItemBar const * const ib, cairo_t *cr,
 
 	/* The widget parameters could be NULL, but if so some themes would emit a warning.
 	 * (Murrine is known to do this: http://bugzilla.gnome.org/show_bug.cgi?id=564410). */
-	gtk_paint_shadow (style,
+	gtk_paint_shadow (widget->style,
 			  gtk_layout_get_bin_window (canvas),
 			  GTK_STATE_NORMAL, shadow,
 			  NULL, widget, "GnmItemBarCell",
@@ -304,7 +305,8 @@ ib_draw_cell (ItemBar const * const ib, cairo_t *cr,
 
 	cairo_save (cr);
 	cairo_clip (cr);
-	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (font_color));
+//	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (font_color));
+	cairo_set_source_rgb (cr, 0., 0., 0.);
 	cairo_translate (cr,
 					 rect->x + (rect->width - PANGO_PIXELS (size.width)) / 2,
 					 rect->y  + (rect->height - PANGO_PIXELS (size.height)) / 2 + ascent);
@@ -783,14 +785,10 @@ colrow_tip_setlabel (ItemBar *ib, gboolean const is_cols, int size_pixels)
 		char *buffer;
 		double const scale = 72. / gnm_app_display_dpi_get (!is_cols);
 		if (is_cols)
-			buffer = g_strdup_printf (ngettext ("Width: %.2f pts (%d pixel)",
-							"Width: %.2f pts (%d pixels)",
-							size_pixels),
+			buffer = g_strdup_printf (_("Width: %.2f pts (%d pixels)"),
 						  scale*size_pixels, size_pixels);
 		else
-			buffer = g_strdup_printf (ngettext ("Height: %.2f pts (%d pixel)",
-							"Height: %.2f pts (%d pixels)",
-							size_pixels),
+			buffer = g_strdup_printf (_("Height: %.2f pts (%d pixels)"),
 						  scale*size_pixels, size_pixels);
 		gtk_label_set_text (GTK_LABEL (ib->tip), buffer);
 		g_free(buffer);
@@ -865,7 +863,7 @@ item_bar_button_pressed (GocItem *item, int button, double x_, double y_)
 	GdkEventButton *event = (GdkEventButton *) goc_canvas_get_cur_event (item->canvas);
 	gint64 x = x_ * item->canvas->pixels_per_unit, y = y_ * item->canvas->pixels_per_unit;
 
-	if (button > 3)
+	if (button > 1)
 		return FALSE;
 
 	if (wbc_gtk_get_guru (wbcg) == NULL)
@@ -878,7 +876,7 @@ item_bar_button_pressed (GocItem *item, int button, double x_, double y_)
 	if (minor_pos < ib->indent)
 		return outline_button_press (ib, element, minor_pos);
 
-	if (button == 3) {
+	if (button == 1) {
 		if (wbc_gtk_get_guru (wbcg) != NULL)
 			return TRUE;
 		/* If the selection does not contain the current row/col
@@ -888,9 +886,18 @@ item_bar_button_pressed (GocItem *item, int button, double x_, double y_)
 			scg_colrow_select (scg, is_cols,
 					   element, event->state);
 
-		scg_context_menu (scg, event, is_cols, !is_cols);
-		return TRUE;
-	} else if (cri != NULL) {
+		guint32 double_click_time;
+
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (canvas)),
+			      "gtk-double-click-time", &double_click_time,
+			      NULL);
+
+		if ((last_click_time_cr + double_click_time) > event->time){
+			scg_context_menu (scg, event, is_cols, !is_cols);
+			return TRUE;
+		}
+		else{
+		//	} else if (cri != NULL) {
 		/*
 		 * Record the important bits.
 		 *
@@ -898,16 +905,17 @@ item_bar_button_pressed (GocItem *item, int button, double x_, double y_)
 		 * we know that we are being resized (used in the
 		 * other event handlers).
 		 */
-		ib->colrow_being_resized = element;
-		ib->resize_start_pos = (is_cols && sheet->text_is_rtl)
-			? start : (start - cri->size_pixels);
-		ib->colrow_resize_size = cri->size_pixels;
+			if (cri != NULL) {
+				ib->colrow_being_resized = element;
+				ib->resize_start_pos = (is_cols && sheet->text_is_rtl)
+					? start : (start - cri->size_pixels);
+				ib->colrow_resize_size = cri->size_pixels;
 
-		if (ib->tip == NULL) {
-			GtkWidget *cw = GTK_WIDGET (canvas);
-			int wx, wy;
-			ib->tip = gnumeric_create_tooltip (cw);
-			colrow_tip_setlabel (ib, is_cols, ib->colrow_resize_size);
+				if (ib->tip == NULL) {
+					GtkWidget *cw = GTK_WIDGET (canvas);
+					int wx, wy;
+					ib->tip = gnumeric_create_tooltip (cw);
+					colrow_tip_setlabel (ib, is_cols, ib->colrow_resize_size);
 			/* Position above the current point for both
 			 * col and row headers.  trying to put it
 			 * beside for row headers often ends up pushing
@@ -915,11 +923,16 @@ item_bar_button_pressed (GocItem *item, int button, double x_, double y_)
 			 * effects on the event stream.  win32 was
 			 * different from X. */
 
-			gnm_canvas_get_position (canvas, &wx, &wy,x, y);
-			gnumeric_position_tooltip (ib->tip,
-						   wx, wy, TRUE);
-			gtk_widget_show_all (gtk_widget_get_toplevel (ib->tip));
+					gnm_canvas_get_position (canvas, &wx, &wy,x, y);
+					gnumeric_position_tooltip (ib->tip,
+							   wx, wy, TRUE);
+					gtk_widget_show_all (gtk_widget_get_toplevel (ib->tip));
+				}
+			}
 		}
+		ib->start_selection = element;
+		gnm_pane_slide_init (pane);
+		last_click_time_cr = event->time;
 	} else {
 		if (wbc_gtk_get_guru (wbcg) != NULL &&
 		    !wbcg_entry_has_logical (wbcg))
@@ -1072,7 +1085,7 @@ item_bar_dispose (GObject *obj)
 	ib_fonts_unref (ib);
 
 	if (ib->tip) {
-		gtk_object_destroy (GTK_OBJECT (ib->tip));
+		gtk_widget_destroy (ib->tip);
 		ib->tip = NULL;
 	}
 
